HonorCapItAll = LibStub('AceAddon-3.0'):NewAddon('HonorCapItAll', 'AceEvent-3.0')
local L = LibStub('AceLocale-3.0'):GetLocale('HonorCapItAll', true)

--   data consists of the relevent currency and quest data we need to calculate
--   honor from them.
--
--   Currency entries consists of a table with the following keys:
--   id = The id of the currency item or Honor or Arena
--   entry_type = 'currency'
--   count =  The current count the character has for that currency
--   honor = The amount of honor the item represents.
--   from = table of the currencies and the purchase cost in them to buy this entry
--
--   Quest entries consists of a table with the following keys:
--   id = The id of the quest
--   entry_type = 'quest'
--   min_level = minimum level you can turn in the queset
--   max_level = maximum level yu can turn in the quest
--   faction = Alliance or Horde
--   items = table of the itemids of things required to complete the quest and the count
--   honor = The amount of honor the quest rewards.
--
--   Note that not every quest needs to go in here, only quests which do not show in
--   your quest log, such as the BG mark repeatables.
--
--   The order of this table matters in two ways.
--   1) It determines the order in which everything is processed.  So that if in the
--      future Blizzards adds a way to receive honor from the same source but to different
--      degrees you can prioritize methods by best honor return.  E.G. the old 2k for all 4
--      battleground tokens vs 300 honor for 1.
--   2) It controls the order in which the items are displayed in the tooltip.  

local data = {
	{ 
		id = 20558, -- Warsong Gulch Mark of Honor
		entry_type = 'currency',
	},
	{
		id = 20559, -- Arathi basin Mark of Honor
		entry_type = 'currency',
	},
	{
		id = 20560, -- Alterac Valey Mark of Honor
		entry_type = 'currency',
	},
	{
		id = 29024, -- Eye of the Storm Mark of Honor
		entry_type = 'currency',
	}, 
	{
		id = 42425, -- Strand of the Ancients Mark of Honor
		entry_type = 'currency',
	}, 
	{
		id = 47395, -- Isle of Conquest Mark of Honor
		entry_type = 'currency',
	},
	{ 
		id=8371, -- Concerted Efforts
		entry_type = 'quest',
		min_level=61,
		max_level=70,
		faction='Alliance',
		items = { [20558]=1, [20559]=1, [20560]=1, [29024]=1 },
		honor=314
	},
	{ 
		id=13478, -- Concerted Efforts
		entry_type = 'quest',
		min_level=71,
		max_level=80,
		faction='Alliance',
		items = { [20558]=1, [20559]=1, [20560]=1, [29024]=1, [42425]=1, [47395]=1 },
		honor=1489
	},
	{ 
		id=8367, -- For Great Honor
		entry_type = 'quest',
		min_level=61,
		max_level=70,
		faction='Horde',
		items = { [20558]=1, [20559]=1, [20560]=1, [29024]=1 },
		honor=314
	}, 
	{
		id=13476, -- For Great Honor
		entry_type = 'quest',
		min_level=71,
		max_level=80,
		items = { [20558]=1, [20559]=1, [20560]=1, [29024]=1, [42425]=1, [47395]=1 },
		faction='Horde',
		honor=1489
	},
	{
		id = 54637, -- Commendation of Service 
		entry_type = 'currency',
		honor = 185,
		from = { [20558]=1, [20559]=1, [20560]=1, [29024]=1, [42425]=1, [47395]=1 },
	},
	{ 
		-- Special entry to trigger the quest log processing.  This entry must be
		-- before anything that is bought with something a quest rewards (e.g.
		-- Commendation of Bravery and Wintergrasp Commendation via Arena Points and
		-- Stone Keeper's Shards respectively).
		id = 'questlog',
		entry_type = 'questlog',
	},
	{
		id = 43228, -- Stone Keeper's Shard
		entry_type = 'currency',
	},
	{
		id = 43589, -- Wintergrsp Mark of Honor
		entry_type = 'currency',
	},
	{
		id = 44115, -- Wintergrasp Commendation
		entry_type = 'currency',
		honor = 2000,
		from = { [43228] = 30, [43589] = 9 }
	},
	{
		id = 45706, -- Commendation of Bravery
		entry_type = 'currency',
		honor = 2000,
		from = { ['Arena'] = 100 }
	},
}
HonorCapItAll.data = data
		
-- Backwards compat to pre 3.3.3 battleground token changes
do
	local wow_version, wow_build, wow_date, wow_interface = GetBuildInfo()
	if tonumber(wow_build) < 11599 then
		table.remove(data, 11) -- Remove the Commendation of Service
	end
end

-- Generator to return the next currency in the data table
local function next_currency(data, i)
	i = next(data, i)
	if i == nil then
		return
	end
	local entry = data[i]
	if entry.entry_type == 'currency' then
		return i, entry
	end
	return next_currency(data, i)
end

-- Generator to return the next quest in the data table
local function next_quest(data, i)
	i = next(data, i)
	if i == nil then
		return
	end
	local entry = data[i]
	if entry.entry_type == 'quest' then
		return i, entry
	end
	return next_quest(data, i)
end

-- Metatable to ensure that a table returns 0 for any missing entries
local zero_mt = {
	__index = function(t, k)
		if k == nil then return end
		t[k] = 0
		return 0
	end,
}

local DATABASE_DEFAULTS = {
	profile = {
		include_questlog = true,
		minimap = {
			hide = false,
		},
		max = {
			[45706] = 5,
			['**'] = 0,
		},
		set_aside = {
			['**'] = 0,
		},
	}
}

-- currency_counts tracks the amount of any given currency we have.
local currency_counts = { }
setmetatable(currency_counts, zero_mt)
HonorCapItAll.currency_counts = currency_counts

-- quest_counts tracks the rewards from the completed quests that are
-- relevent
local quest_counts = { }
HonorCapItAll.quest_counts = quest_counts

-- Keep track of localized quest names. 
local quest_titles = {
	-- Include some English defaults in case the tooltip scanning fails
	-- for some reason.
	[8371]  = 'Concerted Efforts',
	[13478] = 'Concerted Efforts',
	[8367]  = 'For Great Honor',
	[13476] = 'For Great Honor',
}

-- Iterate the currencies and update our counts for them.
local function update_currency_counts()
	for _, entry in next_currency, data do
		local id = entry.id
		currency_counts[id] = GetItemCount(id, true --[[include bank]])
	end

	-- Special casing for Honor and Arena 
	currency_counts['Honor'] = GetHonorCurrency()
	currency_counts['Arena'] = GetArenaCurrency()
end

-- Get the table for the given entry in the quests table or make it
-- if it doesn't exist.  If it exists clear it for a new fill.
local function new_quest_counts_entry(i)
	local entry = quest_counts[i]
	if not entry then
		entry = { }
		setmetatable(entry, zero_mt)
		quest_counts[i] = entry
	else
		wipe(entry)
	end
	return entry
end

-- Return true if the id given is a currency we care about.
local function is_currency(id)
	for _,entry in next_currency, data do
		if entry.id == id then return true end
	end
	return false
end

-- Recursively iterate the quest log
local function iterate_quest_log(position, start, last)
	if not start then start = 1 end
	if not last then last = GetNumQuestLogEntries() end

	for i = start, last do
		local title,_,_,_,_,isCollapsed,isComplete,_,quest_id = GetQuestLogTitle(i)
		if isCollapsed then
			local count = GetNumQuestLogEntries()
			ExpandQuestHeader(i)
			count = GetNumQuestLogEntries() - count
			position = iterate_quest_log(position, i + 1, i + count) 
			CollapseQuestHeader(i)
		elseif isComplete and isComplete > 0 then
			local entry -- table to store our results 

			SelectQuestLogEntry(i)

			-- Look for rewards we care about
			for r = 1, GetNumQuestLogRewards() do
				local _,_,reward_count = GetQuestLogRewardInfo(r)
				local id = tonumber(GetQuestLogItemLink('reward', r):match('item:(%d+):'))
				if is_currency(id) then
					if not entry then
						entry = new_quest_counts_entry(position)
					end
					entry[id] = reward_count
				end
			end

			local honor = GetQuestLogRewardHonor()
			local arena = GetQuestLogRewardArenaPoints()
			if honor > 0 or arena > 0 then
				if not entry then
					entry = new_quest_counts_entry(position) 
				end
				entry.Honor = honor
				entry.Arena = arena
			end

			if entry then -- We found a reward we care about
				entry.quest_id = quest_id
				quest_titles[quest_id] = title
				position = position + 1
			end
		end
	end

	return position
end

-- Update the quest_counts table 
local function update_quest_counts()
	local position = 1
	if HonorCapItAll.db.profile.include_questlog then
		local saved_i = GetQuestLogSelection()
		position = iterate_quest_log(position)
		SelectQuestLogEntry(saved_i)
	end

	-- Clear any unused tables
	for i = position, #quest_counts do
		wipe(quest_counts[i])
	end
end

-- Table we build to keep track of the honor calculations.  This is what we build our
-- tooltip off of.  
--
-- It's indexed with each value set to a table.  Each table has the following keys:
-- src_id = The source item or quest.
-- src_count = The number of the item we have or the number of turnins we can do.
-- src_type = 'quest' or 'currency'
-- dst_id = The item the source converted into or nil if converted to honor
-- dst_count = The number of items the conversion created or nil if honor
-- dst_type = 'quest' or 'currency'
-- rewards = table of currencies and the rewarded value 
local honor_cache = { }
HonorCapItAll.honor_cache = honor_cache

-- Helper function for update_honor_cache().  It sets the specified position in
-- the honor_cache to the passed values.  The last value can be a table with the
-- rewards or you can pass two values of just the honor and the arena points.
-- The table will ignore any non-number values other than Honor or Arena.
local function set_honor_cache_entry(pos, src_id, src_count, src_type, dst_id, dst_count, dst_type, ...)

	-- Deal with getting the variable arguments
	local rewards_table, honor, arena
	local arg1, arg2 = ...
	if arg1 and type(arg1) == 'table' then
		rewards_table = arg1
	else
		honor, arena = arg1, arg2
	end

	-- Create the tables
	local entry = honor_cache[pos]
	if not entry then
		entry = { }
		honor_cache[pos] = entry
	end
	local rewards = entry.rewards
	if not rewards then
		rewards = { }
		setmetatable(rewards, zero_mt)
		entry.rewards = rewards
	else
		-- Already exists so wipe it since we won't necessaryly overwrite
		-- every value.
		wipe(rewards)
	end

	entry.src_id = src_id
	entry.src_count = src_count
	entry.src_type = src_type
	entry.dst_id = dst_id
	entry.dst_count = dst_count
	entry.dst_type = dst_type
	if rewards_table then
		rewards.Honor = rewards_table.Honor
		rewards.Arena = rewards_table.Arena
		for k,v in pairs(rewards_table) do
			if tonumber(k) then -- ignore non-number fields (e.g. quest_id)
				rewards[k] = v
			end
		end
	else
		rewards.Honor = honor
		rewards.Arena = arena
	end
end

local player_faction = UnitFactionGroup('player')
local function update_honor_cache()
	local position = 1
	local spent = { }
	setmetatable(spent, zero_mt)
	local extra = { }
	setmetatable(extra, zero_mt)

	-- Copy the setasides to the spent table, since they're basically the same
	for k,v in pairs(HonorCapItAll.db.profile.set_aside) do
		spent[k] = v
	end

	local player_level = UnitLevel('player')

	for _, entry in ipairs(data) do
		local entry_type = entry.entry_type
		if entry_type == 'currency' then
			local id, honor, arena, from = entry.id, entry.honor, entry.arena, entry.from
			local count = currency_counts[id] - spent[id] + extra[id]
			if count < 0 then count = 0 end
			local max = HonorCapItAll.db.profile.max[id]

			if honor then
				-- Handles the conversion of the a token to honor
				set_honor_cache_entry(position, id, count, entry_type, nil, nil, nil, count * honor)
				spent[id] = spent[id] + count
				position = position + 1
			end

			if from then
				-- Figures out how many of a given token you can purchase
				for from_id, cost in pairs(from) do
					local from_count = currency_counts[from_id] - spent[from_id] + extra[from_id]
					if from_count < 0 then from_count = 0 end
					local bought = math.floor(from_count/cost)
					if max and max > 0 then
						bought = math.min(bought,max - count)
					end
					set_honor_cache_entry(position, from_id, from_count, entry_type, id, bought, entry_type, honor * bought)
					position = position + 1
					spent[from_id] = spent[from_id] + from_count
					count = count + bought
				end
			end
		elseif entry_type == 'quest' then
			if player_faction == entry.faction and player_level >= entry.min_level and player_level <= entry.max_level then
				local items = entry.items
				if items then
					local quest_id = entry.id
					local turnins_possible 
					for item_id, required in pairs(items) do
						local count = math.floor((currency_counts[item_id] - spent[item_id] + extra[item_id]) / required)
						if not turnins_possible or count < turnins_possible then
							turnins_possible = count
						end
					end
					if turnins_possible then
						set_honor_cache_entry(position, quest_id, turnins_possible, entry_type, nil, nil, nil, entry.honor * turnins_possible)
						for item_id, required in pairs(items) do
							spent[item_id] = spent[item_id] + (required * turnins_possible)
						end
						position = position + 1
					end
				end
			end
		elseif entry_type == 'questlog' then
			for _, entry in ipairs(quest_counts) do
				if entry.quest_id ~= 0 then -- quest_id will be zero if this is an unused table
					set_honor_cache_entry(position, entry.quest_id, 1, 'quest', nil, nil, nil, entry)
					position = position + 1

					-- Add any rewards other than the honor to the extra table.
					for k,v in pairs(entry) do
						if tonumber(k) or k == 'Arena' then
							extra[k] = extra[k] + v
						end
					end
				end
			end
		end	
	end

	-- Wipe out any extra entries beyond what we used.
	for i = position, #honor_cache do
		set_honor_cache_entry(i)
	end
end

-- Pretty self explanatory, returns the total honor from our calculations
function HonorCapItAll:GetTotalHonor()
	local total = currency_counts['Honor']
	for _,entry in pairs(honor_cache) do
		total = total + entry.rewards.Honor
	end
	return total
end

-- Build the frame we use to display total honor on the PVPFrame
local PVPFrameHonorCapItAll = CreateFrame('Frame', 'PVPFrameHonorCapItAll', PVPFrameHonor)
PVPFrameHonorCapItAll:SetWidth(75)
PVPFrameHonorCapItAll:SetHeight(20)
PVPFrameHonorCapItAll:EnableMouse(true)
PVPFrameHonorCapItAll.honor = PVPFrameHonorCapItAll:CreateFontString(nil, nil, 'GameFontNormal')
PVPFrameHonorCapItAll.honor:SetAllPoints(PVPFrameHonorCapItAll)

-- Adjust the positioning of the frames to leave room for our text.
local function adjust_pvp_frame()
	PVPFrameHonor:ClearAllPoints()
	PVPFrameHonor:SetPoint('TOPLEFT',PVPFrameBackground,'TOPLEFT',78,-2)
	PVPFrameHonorCapItAll:ClearAllPoints()
	PVPFrameHonorCapItAll:SetPoint('LEFT',PVPFrameHonorPoints,'RIGHT')
	PVPFrameHonorIcon:ClearAllPoints()
	PVPFrameHonorIcon:SetPoint('LEFT',PVPFrameHonorCapItAll,'RIGHT',4,-6)
	PVPFrameHonorCapItAll:Show()
end

-- Set the text on our frame on the PVPFrame.
local function update_pvp_frame()
	PVPFrameHonorCapItAll.honor:SetText(format(' / %7d',HonorCapItAll:GetTotalHonor()))
end

-- Helper function for show_tooltip
local function format_value(value, value_type, value_id)
	if value_type == 'currency' then
		if value_id == 'Honor' then
			return format('%d |TInterface\\TargetingFrame\\UI-PVP-%s:0:0:0:0:64:64:5:37:3:37|t',value,player_faction)
		elseif value_id == 'Arena' then
			return format('%d |TInterface\\PVPFrame\\PVP-ArenaPoints-Icon:0|t',value)
		else
			return format('%d |T%s:0|t',value,GetItemIcon(value_id))
		end
	elseif value_type == 'quest' then
		if value == 1 then
			return quest_titles[value_id]
		else
			return format('%s x%d',quest_titles[value_id],value)
		end
	end
end

-- Just a wrapper for format_value(honor, 'currency', 'Honor') to save some typing
local function format_honor(honor)
	return format_value(honor, 'currency', 'Honor')
end

-- Just a wrapper for format_value(arena, 'currency', 'Arena') to save some typing
local function format_arena(arena)
	return format_value(arena, 'currency', 'Arena')
end

-- Anchoring code borrowed from LibQTip's smart anchor code. 
local function get_tooltip_anchor(frame)
	local x,y = frame:GetCenter()
	if not x or not y then return 'TOPLEFT', 'BOTTOMLEFT' end
	local hhalf = (x > UIParent:GetWidth() * 2/3) and 'RIGHT' or (x < UIParent:GetWidth() / 3) and 'LEFT' or ''
	local vhalf = (y > UIParent:GetHeight() / 2) and 'TOP' or 'BOTTOM'
	return vhalf..hhalf, frame, (vhalf == 'TOP' and 'BOTTOM' or 'TOP')..hhalf
end

local function are_there_rewards(rewards)
	for _,v in pairs(rewards) do
		if v > 0 then return true end
	end
	return false
end

local tooltip_owner
local function show_tooltip(frame)
	if frame then
		tooltip_owner = frame
		if frame == PVPFrameHonorCapItAll then
			-- From the PVPFrame we use the default anchor
			GameTooltip_SetDefaultAnchor(GameTooltip, frame)
		else
			-- Otherwise we figure our own to place it near the triggering window.
			GameTooltip:SetOwner(frame, 'ANCHOR_PRESERVE')
			GameTooltip:ClearAllPoints()
			GameTooltip:SetPoint(get_tooltip_anchor(frame))
		end
	end
	GameTooltip:SetText('HonorCapItAll')
	GameTooltip:AddLine(' ')
	local total_honor = currency_counts['Honor']
	GameTooltip:AddDoubleLine(L['Current Honor'],format_honor(total_honor),1,1,1,1,1,1)
	GameTooltip:AddDoubleLine(L['Current Arena Points'],format_arena(currency_counts['Arena']),1,1,1,1,1,1)
	GameTooltip:AddLine(' ')
	for _,entry in pairs(honor_cache) do
		local rewards = entry.rewards
		if are_there_rewards(rewards) then
			local src_id, src_count, src_type = entry.src_id, entry.src_count, entry.src_type
			local dst_id, dst_count, dst_type = entry.dst_id, entry.dst_count, entry.dst_type
			local honor, arena = rewards.Honor, rewards.Arena
			local left, right

			if src_type then
				left = format_value(src_count,src_type,src_id)
			end
	
			if dst_type then
				left = left .. ' = ' .. format_value(dst_count,dst_type,dst_id)
			end

			left = left .. ' = '

			for k,v in pairs(rewards) do
				if tonumber(k) then -- skip Honor and Arena and do them later
					if not right then
						right = format_value(v,'currency',k)
					else
						right = right .. ' ' .. format_value(v,'currency',k)
					end
				end
			end
			if arena > 0  then
				if not right then
					right = format_arena(arena)
				else
					right = right .. ' ' .. format_arena(arena)
				end
			end
			if honor > 0 then
				if not right then
					right = format_honor(honor)
				else
					right = right .. ' ' .. format_honor(honor)
				end
			end

			total_honor = total_honor + honor
			GameTooltip:AddDoubleLine(left,right,1,1,1,1,1,1)
		end
	end
	
	GameTooltip:AddLine(' ')
	GameTooltip:AddDoubleLine(L['Total Honor'],format_honor(total_honor),1,1,1,1,1,1)

	if tooltip_owner ~= PVPFrameHonorCapItAll then
		-- LDB so add the click info
		GameTooltip:AddLine(' ')
		GameTooltip:AddLine(L['Right-click to open the config window'])
	end

	GameTooltip:Show()
end

local function hide_tooltip()
	GameTooltip:Hide()
end

PVPFrameHonorCapItAll:SetScript('OnEnter',show_tooltip)
PVPFrameHonorCapItAll:SetScript('OnLeave',hide_tooltip)

local function update_tooltip()
	if GameTooltip:GetOwner() == tooltip_owner then
		show_tooltip()
	end
end

-- Setup the LDB support.
local ldb = LibStub('LibDataBroker-1.1',true)
local ldb_icon = LibStub('LibDBIcon-1.0', true)
local ldb_obj
if ldb then
	ldb_obj = ldb:NewDataObject('HonorCapItAll',
			{
			type = 'data source',
			text = 'Initializing...',
			icon = 'Interface\\Icons\\Spell_Holy_DivinePurpose', 
			suffix = format('|TInterface\\TargetingFrame\\UI-PVP-%s:0:0:0:0:64:64:5:37:3:37|t',player_faction),
			label = L['Total Honor'],
			OnEnter = show_tooltip,
			OnLeave = hide_tooltip,
			OnClick = function (self, button)
				if button == 'RightButton' then
					InterfaceOptionsFrame_OpenToCategory('HonorCapItAll')
				end
			end,
		}
	)
end

local function update_ldb()
	if not ldb_obj then return end
	ldb_obj.value = HonorCapItAll:GetTotalHonor()
	ldb_obj.text = format_honor(ldb_obj.value)
end

local function update_ldb_icon()
	if HonorCapItAll.db.profile.minimap.hide then
		ldb_icon:Hide('HonorCapItAll')
	else
		ldb_icon:Show('HonorCapItAll')
	end
end

local function build_config()
	local AceConfig = LibStub('AceConfig-3.0')
	local AceConfigDialog = LibStub('AceConfigDialog-3.0')

	local max_order = 1
	local max_group = {
		desc = {
			type = 'description',
			name = L['Set the maximum number of this item to buy, including ones that you already own.  This is useful for limiting the Commendation of Bravery at the end of the arena season since it is Unique(5).  If the maximum is set to zero no limit is enforced.'],
			order = max_order,
		},
		spacer = {
			type = 'description',
			name = '',
			order = max_order + 1,
		},
	}
	max_order = max_order + 2

	local set_aside_order = 1
	local set_aside_group = {
		desc = {
			type = 'description',
			name = L['Set aside some of a given currency.  Useful if you wish to account for planned spending on items that you might buy with these currencies other than converting them to honor.  You could use it to prevent a currency from buying any tokens, for example setting the arena point set aside to a really high number can be used to prevent your arena points from being calculated to use for honor.'],
			order = set_aside_order,
		},
		spacer = {
			type = 'description',
			name = '',
			order = set_aside_order + 1,
		},
	}
	set_aside_order = set_aside_order + 2

	local function name(info)
		local id = info[#info]
		local icon
		if id == 'Arena' then
			icon = 'Interface\\PVPFrame\\PVP-ArenaPoints-Icon'
		else
			icon = GetItemIcon(id)
		end
		return format(' |T%s:32|t',icon)
	end

	local function desc(info)
		local id = info[#info]
		if id == 'Arena' then
			return ARENA_POINTS
		else
			return GetItemInfo(id)
		end
	end

	local function get(info)
		local table = info[#info-1]
		local id = tonumber(info[#info]) or info[#info]
		return tostring(HonorCapItAll.db.profile[table][id])
	end

	local function set(info, value)
		local table = info[#info-1]
		local id = tonumber(info[#info]) or info[#info]
		HonorCapItAll.db.profile[table][id] = tonumber(value)
		update_honor_cache()
		update_ldb()
		update_pvp_frame()
		update_tooltip()
	end

	local function validate(info, value)
		if not tonumber(value) then
			return L['Must be a number.']
		end
		return true
	end

	local max = {
		name = name,
		desc = desc,
		type = 'input',
		get = get,
		set = set,
		validate = validate,
		width = 'half',
	}
	local set_aside = {
		name = name,
		desc = desc,
		type = 'input',
		get = get,
		set = set,
		validate = validate,
		width = 'half',
	}

	local arena = { }
	for k,v in pairs(set_aside) do
		arena[k] = v
	end
	arena.order = set_aside_order
	set_aside_order = set_aside_order + 1
	set_aside_group['Arena'] = arena
	
	for _,entry in next_currency, data do
		if entry.from then
			local config_entry = { }
			for k,v in pairs(max) do
				config_entry[k] = v
			end
			config_entry.order = max_order
			max_order = max_order + 1
			max_group[tostring(entry.id)] = config_entry
		else
			local config_entry = { }
			for k,v in pairs(set_aside) do
				config_entry[k] = v
			end
			config_entry.order = set_aside_order
			set_aside_order = set_aside_order + 1
			set_aside_group[tostring(entry.id)] = set_aside
		end
	end

	local config = {
		name = 'HonorCapItAll',
		type = 'group',
		args = {
			max = {
				name = L['Maximum'],
				type = 'group',
				inline = true,
				args = max_group,
				order = 1,
			},
			set_aside = {
				name = L['Set Aside'],
				type = 'group',
				inline = true,
				args = set_aside_group,
				order = 2,
			},
			misc = {
				name = L['Miscellaneous'],
				type = 'group',
				inline = true,
				order = 3,
				args = {
					minimap = {
						type = 'toggle',
						name = L['Hide Minimap Icon'],
						desc = L['Hide the minimap icon for this addon.'],
						get = function(info)
							return HonorCapItAll.db.profile.minimap.hide
						end,
						set = function(info, value)
							HonorCapItAll.db.profile.minimap.hide = value
							update_ldb_icon()
						end,
						hidden = function(info)
							return not ldb_icon
						end,
						order = 1,
					},
					include_questlog = {
						type = 'toggle',
						name = L['Include Completed Quests'],
						desc = L['Include completed quests from your quest log in the results.'],
						order = 2,
						get = function(info)
							return HonorCapItAll.db.profile.include_questlog
						end,
						set = function(info, value)
							HonorCapItAll.db.profile.include_questlog = value
							update_quest_counts()
							update_honor_cache()
							update_ldb()
							update_pvp_frame()
							update_tooltip()
						end,
						width = 'double',
					},
				},
			},
		},
	}

	local profile = LibStub('AceDBOptions-3.0'):GetOptionsTable(HonorCapItAll.db)

	AceConfig:RegisterOptionsTable('HonorCapItAll',config)
	AceConfigDialog:AddToBlizOptions('HonorCapItAll','HonorCapItAll')
	AceConfig:RegisterOptionsTable('HonorCapItAll.profile', profile)
	AceConfigDialog:AddToBlizOptions('HonorCapItAll.profile', profile.name, 'HonorCapItAll')


	-- no longer need ourself
	build_config = nil
end

function HonorCapItAll:OnInitialize()
	-- Fill quest_titles the quests in our data table.
	local tt = CreateFrame('GameTooltip')
	local tt_left = tt:CreateFontString(nil, nil, 'GameTooltipText')
	local tt_right = tt:CreateFontString(nil, nil, 'GameTooltipText')
	tt:AddFontStrings(tt_left, tt_right)
	tt:SetOwner(UIParent, 'ANCHOR_NONE')
	for _,entry in next_quest, data do
		local id = entry.id
		tt:SetHyperlink('quest:' .. id)
		local title = tt_left:GetText()
		if title then
			quest_titles[id] = tt_left:GetText()
		end
	end

	local db = LibStub('AceDB-3.0'):New('HonorCapItAllDB', DATABASE_DEFAULTS, 'Default')
	DATABASE_DEFAULTS = nil
	self.db = db

	db.RegisterCallback(self, 'OnProfileChanged')
	db.RegisterCallback(self, 'OnProfileCopied', 'OnProfileChanged')
	db.RegisterCallback(self, 'OnProfileReset', 'OnProfileChanged')

	if ldb_icon and ldb_obj then
		ldb_icon:Register('HonorCapItAll', ldb_obj, self.db.profile.minimap)
	end

	build_config()
end

function HonorCapItAll:OnEnable()
	-- Events that cause us to update the currency table
	self:RegisterEvent('CURRENCY_DISPLAY_UPDATE')
	self:RegisterEvent('KNOWN_CURRENCY_TYPES_UPDATE','CURRENCY_DISPLAY_UPDATE')
	self:RegisterEvent('BAG_UPDATE','CURRENCY_DISPLAY_UPDATE')
	self:RegisterEvent('PLAYERBANKSLOTS_CHANGED','CURRENCY_DISPLAY_UPDATE')

	-- Events for the quest table
	self:RegisterEvent('QUEST_LOG_UPDATE')

	-- Events that simply force a recalculate of honor
	self:RegisterEvent('PLAYER_LEVEL_UP')

	-- Fill our tables
	update_currency_counts()
	update_quest_counts()
	update_honor_cache()
	update_ldb()
	update_tooltip()

	adjust_pvp_frame()
	update_pvp_frame()
end

function HonorCapItAll:OnProfileChanged()
	update_honor_cache()
	update_ldb()
	update_ldb_icon()
	update_pvp_frame()
	update_tooltip()
end

function HonorCapItAll:PLAYER_LEVEL_UP()
	update_honor_cache()
	update_ldb()
	update_pvp_frame()
	update_tooltip()
end

function HonorCapItAll:QUEST_LOG_UPDATE()
	update_quest_counts()
	update_honor_cache()
	update_ldb()
	update_pvp_frame()
	update_tooltip()
end

function HonorCapItAll:CURRENCY_DISPLAY_UPDATE()
	update_currency_counts()
	update_honor_cache()
	update_ldb()
	update_pvp_frame()
	update_tooltip()
end



